.TH coils 1 "07 June 1996" Unix/Linux
.SH NAME
coils \- generate coils, springs and the like 
.SH SYNOPSIS
.B coils
[
.I \-as?
]
[
.I \-r rotations
]
[
.I \-t tightness
]
[
.I \-w radius
]
[
.I \-W radius
]
[
.I \-f filename
]
[
.I \-N object
]
[
.I \-o output_format
]
.SH DESCRIPTION
The coils program produces an output file suitable for use with rendering
programs that model coils or springs.  Currently only POV-Ray include files
are supported for output formats.
.PP
The final object is centered about the origin.  The coordinates of both
ends of the coil are provided in the comment section of the output file.
No textures are provided.
.SH OPTIONS
.TP 8
.BI \-r " rotations" 
Specifies the number of 360 degree rotations in the coil.
.TP 8
.BI \-t " tightness" 
Specifies the  offset of end of rotation from 
start of rotation along rotation axis.
.TP 8
.BI \-w " radius" 
Specifies the radius of the coil strand (minor radius of torus).
.TP 8
.BI \-W " radius" 
Specifies the radius of a single rotation (major radius of torus).
.TP 8
.BI \-s 
Use the sturm option for all torii.
.TP 8
.BI \-a 
Append to output file, if it already exists.
.TP 8
.BI \-f " filename"
Filename to write output to (default: coils.inc).
.TP 8
.BI \-N " object"
Name of the object (default: coil).
.TP 8
.BI \-o " type"
Output format type. This can be one of the following: 
.in +3
1 (for POV-Ray - only format currently supported) 
.br
Currently this option does not need to be specified since it defaults
to POV-Ray.
.in -3
.SH EXAMPLE USAGE
Suppose you want to create a coiled strand around a barrel, where each
rotation of the coil touches its adjoining neighbor.  
Suprisingly,
thats exactly why I wrote this little ditty.  How would you do this?  You
need to know the following things:
.PP
1. The number of rotations the strand will make.  10 rotations should be
enough for this example.
.PP
2. The radius of the strand.
Lets say our barrel is 8 units long and
will have 10 rotations in its coil, we'll need 8/(10/2)=1.6 units.
Remember, this is the minor radius value for the torii that will make up
the coil.
.PP
3. The radius of a complete rotation of the strand.
We'll make our barrel 4 units in diameter, so the radius of a complete 
rotation of the coil around the barrel would be 2+1.6=3.6.
.PP
4. The distance between the apex of one rotation of the strand to the apex of
the next rotation of the strand.  This is the tightness.  
This is simply 2 times the radius of the strand, since each rotation touches
its neighbor.  For springs this value would be greater than 2 times the
strand radius, so there is empty space between each rotation.
.PP
We also want two of these coils, with unique names but in the same include file.
The include file should be called "mycoils.inc".
We'll need to run the coils command twice.  The first invokation would be
as follows:
.sp 2
.in +3
coils -r 10 -w 1.6 -W 3.6 -t 3.2 -f mycoils.inc -N coil1
.in -3
.PP
The next invokation needs to tell coils program to append to an existing
file.  We do this with the \-a option:
.sp 2
.in +3
coils -a -r 10 -w 1.6 -W 3.6 -t 3.2 -f mycoils.inc -N coil2
.in -3
.PP
Viola!  You now have two coils named coil1 and coil2 that can be used
within your POV-Ray scene file.
.SH HOW DOES IT WORK
The coils program creates two half torii, each with one end anchored on the
origin.  It then rotates each of these an appropriate amount so that the
unanchored end is half the tightness distance.  The two half torii are then
connected to create a single full rotation.  This object is then used to
create a coil of how ever many strands the user requires.
.SH "SEE ALSO"
gears(1)
.SH BUGS
It was quick and dirty and if you do stupid things it breaks.  So, don't do
stupid things.
.PP
The command line options use atof() to convert the input to a double.  This
only provides 6 digit precision unfortunately.  I'm too lazy to convert
this to a long double function so if you need greater precision you'll need
to replace the atof() calls to your own version.
.PP
Torii are notorious for slowing the processing of an image.  Although a
bounding box around the coil helps speed processing, you should be aware
that the coil itself will be quite slow to render, especially if you
include a large number of rotations.
.SH AUTHOR
Copyright 1996 by Michael J. Hammel <mjhammel@csn.net>
