/* $Id$ */

/*========================================================================
 *  Copyright (c) 1996 Michael J. Hammel <mjhammel@csn.net>
 *========================================================================
 *              FILE NAME: box.c
 *            DESCRIPTION: builds box shaped frames
 *      DEFINED CONSTANTS: 
 *       TYPE DEFINITIONS: 
 *      MACRO DEFINITIONS: 
 *       GLOBAL VARIABLES: 
 *       PUBLIC FUNCTIONS: 
 *      PRIVATE FUNCTIONS: 
 *  SOFTWARE DEPENDENCIES: 
 *  HARDWARE DEPENDENCIES: 
 *                  NOTES: 
 *
 * SPECIAL CONSIDERATIONS:
 * Set tabstops to 3 to make this file more readable.
 *
 *========================================================================
 *
 * MODIFICATION HISTORY:
 * $Log$
 *
 *========================================================================*/

/* === System Headers === */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>


/* === Project Headers === */
#include "genf.h"

/* === external routines === */
extern char *utilGetWordA(char *line, char stop);
extern char *utilGetValue(char *name, struct entry *head);
extern int  utilIsNumber(char *string);
extern int  utilIsFloat(char *string);
extern char *utilStrupr(char *string);
extern int  utilReadConfigFile( char *config_file, struct entry **head );
extern void PrintHeader();

/* === Public routine prototypes === */

/* === Private routine prototypes === */

/* === Global Variables === */
extern char *utilError;
extern char ErrorBuf[255];

extern char	ProgName[255];
extern char	OutFileName[128];
extern FILE	*outfile;

/* character strings */
extern char	FrameShape[128], FrameJointObject[128], FrameTexture[128];

/* numeric values */
extern int		FrameId, FrameElements, FramePoints1, FramePoints2,
					FrameCrossOffset;
extern double	FrameRadius1, FrameRadius2, FrameLength, 
					FrameBoxWidth, FrameBoxHeight, FrameRodRadius;

/* booleans */
extern int	FrameCrossEnds, FrameCrossLength, FrameJoinLength;

/*========================================================================
 *	Name:			BuildBox
 *	Prototype:	void BuildBox()
 *
 *	Description:
 *		builds a box shaped wire frame
 *
 *	Input Arguments:
 *		
 *	Return Values:
 *
 *	Notes:
 *
 *========================================================================*/

void
BuildBox()
{
	Vertices	lower_vertices[4];
	Vertices	upper_vertices[4];

	double	xoffset;
	int		i,j;

	char		buf[128];

	/*
	 * print a header in the output file
	 */
	sprintf( buf, "Box Frame Object: %s_box%d", ProgName, FrameId );
	PrintHeader( buf );


	/*
	 * Compute end points, centering the box at the origin around the x axis
	 */
	for ( i=0; i<4; i++ )		/* X axis */
	{
		lower_vertices[i].x = (double)(0.0-((double)FrameLength/2.0));
		upper_vertices[i].x = (double)(0.0+((double)FrameLength/2.0));
	}
	for ( i=0; i<4; i++ )		/* Y axis */
	{
		switch (i)
		{
			case 0:
			case 1:
				lower_vertices[i].y = upper_vertices[i].y = 
					(double)(0.0-((double)FrameBoxHeight/2.0));
				break;
			case 2:
			case 3:
				lower_vertices[i].y = upper_vertices[i].y = 
					(double)(0.0+((double)FrameBoxHeight/2.0));
				break;
		}
	}
	/* Z axis */
	lower_vertices[0].z = upper_vertices[0].z = 
	lower_vertices[3].z = upper_vertices[3].z = 0.0-((double)FrameBoxWidth/2.0);
	lower_vertices[1].z = upper_vertices[1].z = 
	lower_vertices[2].z = upper_vertices[2].z = 0.0+((double)FrameBoxWidth/2.0);


	/*
	 * Create a union for the squares at both ends of the box
	 */
	(void) fprintf(outfile, "// Left end of box\n" );
	fprintf( outfile, "#declare %s_box_lower_end%d = union {\n", ProgName, FrameId );
	for ( i=0; i<4; i++)
	{
		if ( i<3 )
			fprintf( outfile, "   cylinder { <%f, %f, %f>, <%f, %f, %f>, %f }\n", 
				lower_vertices[i].x, lower_vertices[i].y, lower_vertices[i].z,
				lower_vertices[i+1].x, lower_vertices[i+1].y, lower_vertices[i+1].z,
				FrameRodRadius);
		else
			fprintf( outfile, "   cylinder { <%f, %f, %f>, <%f, %f, %f>, %f }\n", 
				lower_vertices[i].x, lower_vertices[i].y, lower_vertices[i].z,
				lower_vertices[0].x, lower_vertices[0].y, lower_vertices[0].z,
				FrameRodRadius);
	}
	fprintf( outfile, "   }\n" );
	(void) fprintf(outfile, "// Right end of box\n" );
	fprintf( outfile, "#declare %s_box_upper_end%d = union {\n", ProgName, FrameId );
	for ( i=0; i<4; i++)
	{
		if ( i<3 )
			fprintf( outfile, "   cylinder { <%f, %f, %f>, <%f, %f, %f>, %f }\n", 
				upper_vertices[i].x, upper_vertices[i].y, upper_vertices[i].z,
				upper_vertices[i+1].x, upper_vertices[i+1].y, upper_vertices[i+1].z,
				FrameRodRadius);
		else
			fprintf( outfile, "   cylinder { <%f, %f, %f>, <%f, %f, %f>, %f }\n", 
				upper_vertices[i].x, upper_vertices[i].y, upper_vertices[i].z,
				upper_vertices[0].x, upper_vertices[0].y, upper_vertices[0].z,
				FrameRodRadius);
	}
	fprintf( outfile, "   }\n" );


	/* Cross the ends, if requested */
	if ( FrameCrossEnds )
	{
		(void) fprintf(outfile, "// Cross Left end of box\n" );
		fprintf( outfile, "#declare %s_box_cross_lower_end%d = union {\n", ProgName, FrameId );
		fprintf( outfile, "   cylinder { <%f, %f, %f>, <%f, %f, %f>, %f }\n", 
			lower_vertices[0].x, lower_vertices[0].y, lower_vertices[0].z,
			lower_vertices[2].x, lower_vertices[2].y, lower_vertices[2].z,
			FrameRodRadius);
		fprintf( outfile, "   cylinder { <%f, %f, %f>, <%f, %f, %f>, %f }\n", 
			lower_vertices[1].x, lower_vertices[1].y, lower_vertices[1].z,
			lower_vertices[3].x, lower_vertices[3].y, lower_vertices[3].z,
			FrameRodRadius);
		fprintf( outfile, "   }\n" );
		(void) fprintf(outfile, "// Cross Right end of box\n" );
		fprintf( outfile, "#declare %s_box_cross_upper_end%d = union {\n", ProgName, FrameId );
		fprintf( outfile, "   cylinder { <%f, %f, %f>, <%f, %f, %f>, %f }\n", 
			upper_vertices[0].x, upper_vertices[0].y, upper_vertices[0].z,
			upper_vertices[2].x, upper_vertices[2].y, upper_vertices[2].z,
			FrameRodRadius);
		fprintf( outfile, "   cylinder { <%f, %f, %f>, <%f, %f, %f>, %f }\n", 
			upper_vertices[1].x, upper_vertices[1].y, upper_vertices[1].z,
			upper_vertices[3].x, upper_vertices[3].y, upper_vertices[3].z,
			FrameRodRadius);
		fprintf( outfile, "   }\n" );
	}

	/* Join the ends, if requested */
	if ( FrameJoinLength )
	{
		(void) fprintf(outfile, "// Join ends of box\n" );
		fprintf( outfile, "#declare %s_box_cross_join%d = union {\n", ProgName, FrameId );
		for ( i=0; i<4; i++ )
			fprintf( outfile, "   cylinder { <%f, %f, %f>, <%f, %f, %f>, %f }\n", 
				lower_vertices[i].x, lower_vertices[i].y, lower_vertices[i].z,
				upper_vertices[i].x, upper_vertices[i].y, upper_vertices[i].z,
				FrameRodRadius);
		fprintf( outfile, "   }\n" );
	}

	/* Cross the length, if requested */
	if ( FrameCrossLength )
	{
		(void) fprintf(outfile, "// Cross Length of box\n" );
		fprintf( outfile, "#declare %s_box_cross_length%d = union {\n", ProgName, FrameId );
		for ( i=0,j=1; i<4; i++,j++)
		{
			if ( i==3 ) j=0;
			fprintf( outfile, "   cylinder { <%f, %f, %f>, <%f, %f, %f>, %f }\n", 
				lower_vertices[i].x, lower_vertices[i].y, lower_vertices[i].z,
				upper_vertices[j].x, upper_vertices[j].y, upper_vertices[j].z,
				FrameRodRadius);
		}
		for ( i=3,j=2; i>-1; i--,j--)
		{
			if ( i==0 ) j=3;
			fprintf( outfile, "   cylinder { <%f, %f, %f>, <%f, %f, %f>, %f }\n", 
				lower_vertices[i].x, lower_vertices[i].y, lower_vertices[i].z,
				upper_vertices[j].x, upper_vertices[j].y, upper_vertices[j].z,
				FrameRodRadius);
		}
		fprintf( outfile, "   }\n" );
	}



	/*
	 * Combine all pieces into a single union
	 */
	if ( FrameElements == 1 )
	{
		(void) fprintf(outfile, "// Combined box\n" );
		fprintf( outfile, "#declare %s_box%d = union {\n", ProgName, FrameId );
	}
	else
	{
		(void) fprintf(outfile, "// single element \n" );
		fprintf( outfile, "#declare %s_box_element%d = union {\n", ProgName, FrameId );
	}

	fprintf( outfile, "   object { %s_box_lower_end%d }\n", ProgName, FrameId );
	fprintf( outfile, "   object { %s_box_upper_end%d }\n", ProgName, FrameId );
	if ( FrameCrossEnds )
	{
		fprintf( outfile, "   object { %s_box_cross_lower_end%d }\n", ProgName, FrameId );
		fprintf( outfile, "   object { %s_box_cross_upper_end%d }\n", ProgName, FrameId );
	}
	if ( FrameJoinLength )
		fprintf( outfile, "   object { %s_box_cross_join%d }\n", ProgName, FrameId );
	if ( FrameCrossLength )
		fprintf( outfile, "   object { %s_box_cross_length%d }\n", ProgName, FrameId );


	/* 
	 * Add the Joint Objects to the vertices, if requested
	 */
	if ( strcmp ( FrameJointObject, GENF_DEFAULT_JOINT_OBJECT ) != 0 )
	{
		(void) fprintf(outfile, "// Add Joint Objects to the vertices\n" );
		for ( i=0; i<4; i++)
		{
			fprintf( outfile, "   object { %s translate <%f, %f, %f> }\n", 
						FrameJointObject, lower_vertices[i].x, 
						lower_vertices[i].y, lower_vertices[i].z );
			fprintf( outfile, "   object { %s translate <%f, %f, %f> }\n", 
						FrameJointObject, upper_vertices[i].x, 
						upper_vertices[i].y, upper_vertices[i].z );
		}
	}

	/* Texture it, if its the only element */
	if ( FrameElements == 1 )
		fprintf( outfile, "   texture { %s }\n", FrameTexture );
	
	fprintf( outfile, "   }\n" );

	/*
	 * if more than one element has been requested, make a union
	 * of multiple elements
	 */
	if ( FrameElements > 1 )
	{
		(void) fprintf(outfile, "// Combined box\n" );
		fprintf( outfile, "#declare %s_box%d = union {\n", ProgName, FrameId );

		/* compute the translation offsets */
		for (i=FrameElements; i>0; i-=2)
		{
			/* put an element in the union at the computed offset */
			xoffset = (double)(i-1.0)/2.0;
			fprintf( outfile, 
				"   object { %s_box_element%d translate <%f, 0, 0>}\n", 
						ProgName, FrameId, (double)(-1.0*xoffset*FrameLength) );
			fprintf( outfile, 
				"   object { %s_box_element%d translate <%f, 0, 0>}\n", 
						ProgName, FrameId, (double)(xoffset*FrameLength) );
		}

		/* texture the complete object */
		fprintf( outfile, "   texture { %s }\n", FrameTexture );
		fprintf( outfile, "   }\n" );
	}

	(void) fclose(outfile);
}

