/* $Id$ */

/*========================================================================
 *  Copyright (c) 1996 Michael J. Hammel <mjhammel@csn.net>
 *========================================================================
 *              FILE NAME: cone.c
 *            DESCRIPTION: builds cone shaped frames
 *      DEFINED CONSTANTS: 
 *       TYPE DEFINITIONS: 
 *      MACRO DEFINITIONS: 
 *       GLOBAL VARIABLES: 
 *       PUBLIC FUNCTIONS: 
 *      PRIVATE FUNCTIONS: 
 *  SOFTWARE DEPENDENCIES: 
 *  HARDWARE DEPENDENCIES: 
 *                  NOTES: 
 *
 * SPECIAL CONSIDERATIONS:
 * Set tabstops to 3 to make this file more readable.
 *
 *========================================================================
 *
 * MODIFICATION HISTORY:
 * $Log$
 *
 *========================================================================*/

/* === System Headers === */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <math.h>


/* === Project Headers === */
#include "genf.h"

/* === external routines === */
extern char *utilGetWordA(char *line, char stop);
extern char *utilGetValue(char *name, struct entry *head);
extern int  utilIsNumber(char *string);
extern int  utilIsFloat(char *string);
extern char *utilStrupr(char *string);
extern int  utilReadConfigFile( char *config_file, struct entry **head );
extern void PrintHeader();

/* === Public routine prototypes === */

/* === Private routine prototypes === */

/* === Global Variables === */
extern char *utilError;
extern char ErrorBuf[255];

extern char	ProgName[255];
extern char	OutFileName[128];
extern FILE	*outfile;

/* character strings */
extern char	FrameShape[128], FrameJointObject[128], FrameTexture[128];
extern char	FrameConeEndShape[128];

/* numeric values */
extern int		FrameId, FrameElements, FramePoints1, FramePoints2,
					FrameCrossOffset;
extern double	FrameRadius1, FrameRadius2, FrameLength, 
					FrameBoxWidth, FrameBoxHeight, FrameRodRadius;
extern double	FrameConeEndThickness;

/* booleans */
extern int	FrameCrossEnds, FrameCrossLength, FrameJoinLength;



/*========================================================================
 *	Name:			BuildCone
 *	Prototype:	void BuildCone()
 *
 *	Description:
 *		builds a cone shaped wire frame
 *
 *	Input Arguments:
 *		
 *	Return Values:
 *
 *	Notes:
 *
 *========================================================================*/

void
BuildCone()
{
	ConePoints	*link, *link1, *link2, *lastlink, *linkhead1, *linkhead2;

	double		offset, thickness1, thickness2, pointx;
	double		theta1, theta2;
	double		xoffset;

	int			i,j,k;
	char			buf[256];

	/*
	 * compute the centers of the ends of the cone
	 */
	offset = FrameLength/2.0;

	/*
	 * compute the angle (in radians) of equadistant pieces of the rim
	 * based on the number of points supplied by the user (or the defaults).
	 */
	theta1 = 360/FramePoints1*RADIANS;

	/*
	 * Create a link list of vectors for the joint spots on both
	 * rims.  We do this as if the rim was lying on the XY plane.
	 */
	lastlink = linkhead1 = linkhead2 = NULL;
	for ( i=0; i<FramePoints1; i++ )
	{
		
		/* allocate a link */
		link = (ConePoints *)malloc ( sizeof (ConePoints) );
		if ( linkhead1 == NULL )
			linkhead1 = link;

		/* fill in the link structure */
		link->x = -1.0*FrameLength/2.0;
		link->y = FrameRadius1 * sin( i*theta1 );
		link->z = FrameRadius1 * cos( i*theta1 );

		/* add it to the list */
		if ( lastlink != NULL )
			lastlink->next = link;
		lastlink = link;
	}

	/*
	 * If we're not joining the length, we can have different number of 
	 * points on the ends.  If we are joining the length, we force
	 * the same number of points to make things easier.
	 */
	if ( !FrameJoinLength )
	{
		theta2 = 360/FramePoints2*RADIANS;
		lastlink = NULL;
		for ( i=0; i<FramePoints2; i++ )
		{
			/* allocate a link */
			link = (ConePoints *)malloc ( sizeof (ConePoints) );
			if ( linkhead2 == NULL )
				linkhead2 = link;
	
			/* fill in the link structure */
			link->x = FrameLength/2.0;
			link->y = FrameRadius2 * sin( i*theta2 );
			link->z = FrameRadius2 * cos( i*theta2 );
			link->next = NULL;
	
			/* add it to the list */
			if ( lastlink != NULL )
				lastlink->next = link;
			lastlink = link;
		}
	}
	else
	{
		theta2 = theta1;
		lastlink = NULL;
		for ( i=0; i<FramePoints1; i++ )
		{
			/* allocate a link */
			link = (ConePoints *)malloc ( sizeof (ConePoints) );
			if ( linkhead2 == NULL )
				linkhead2 = link;
	
			/* fill in the link structure */
			link->x = FrameLength/2.0;
			link->y = FrameRadius2 * sin( i*theta2 );
			link->z = FrameRadius2 * cos( i*theta2 );
	
			/* add it to the list */
			if ( lastlink != NULL )
				lastlink->next = link;
			lastlink = link;
		}
	}



	/*
	 * print a header in the output file
	 */
	sprintf( buf, "Cone Frame Object: %s_cone%d", ProgName, FrameId );
	PrintHeader( buf );


	/*
	 * Create a union for the rims at both ends of the cone
	 */
	(void) fprintf(outfile, "// Rims of cone\n" );
	fprintf( outfile, "#declare %s_cone_rims%d = union {\n", ProgName, FrameId );
	if (strcmp(FrameConeEndShape, "CYLINDER") == 0 )
	{

		/*
		 * If we're using Cylinders for the rims, we cut out a slightly
		 * smaller version of the cylinder from the original to create a 
		 * rim.
		 */
		pointx = FrameConeEndThickness/2.0;
		if ( (thickness1 = FrameRadius1 - FrameConeEndThickness) <= 0 )
			thickness1 = .0001;
		if ( (thickness2 = FrameRadius2 - FrameConeEndThickness) <= 0 )
			thickness2 = .0001;

		fprintf ( outfile, "   difference { \n ");
		fprintf ( outfile, "     cylinder { <%f,0,0>, <%f,0,0>, %f translate <%f,0,0> } \n ",
						-1.0*pointx, pointx, FrameRadius1, -1.0*offset );
		fprintf ( outfile, "     cylinder { <%f,0,0>, <%f,0,0>, %f translate <%f,0,0> } \n ",
						-1.0*(pointx+.01), pointx+.01, thickness1, -1.0*offset );
		fprintf( outfile, "   }\n" );

		fprintf ( outfile, "   difference { \n ");
		fprintf ( outfile, "     cylinder { <%f,0,0>, <%f,0,0>, %f translate <%f,0,0> } \n ",
						-1.0*pointx, pointx, FrameRadius2, offset );
		fprintf ( outfile, "     cylinder { <%f,0,0>, <%f,0,0>, %f translate <%f,0,0> } \n ",
						-1.0*(pointx+.01), pointx+.01, thickness2, offset );
		fprintf( outfile, "   }\n" );
	}
	else
	{
		/*
		 * Use a torus shape for the rims
		 */
		pointx = FrameConeEndThickness/2.0;
		fprintf ( outfile, "     torus { %f, %f rotate z*90 translate <%f,0,0> } \n ",
						FrameRadius1, pointx, -1.0*offset );
		fprintf ( outfile, "     torus { %f, %f rotate z*90 translate <%f,0,0> } \n ",
						FrameRadius2, pointx, offset );
	}
	fprintf( outfile, "   }\n" );


	/*
	 * Cross the ends, if requested
	 * This is pretty hokey, I think.  But I said I'd put it in, so I am.
	 */
	if ( FrameCrossEnds )
	{
		(void) fprintf(outfile, "// Cross ends of cone\n" );
		fprintf( outfile, "#declare %s_cone_cross_leftend%d = union {\n", 
			ProgName, FrameId );
		link1 = linkhead1;
		for (i=1; i<FramePoints1+1, link1!=NULL; i++ )
		{
			/* Compute the offset joint point */
			if ( (j = i + FrameCrossOffset) > FramePoints1 )
			{
				j = j - FramePoints1;
			}
			k = 1;
			link2 = linkhead1;
			while ( k < j )
			{
				link2 = link2->next;
				k++;
			}
			
			fprintf ( outfile, "     cylinder { <%f,%f,%f>, <%f,%f,%f>, %f } \n",
					link1->x, link1->y, link1->z, link2->x, link2->y, link2->z,
					FrameRodRadius);
			link1 = link1->next;
		}
		fprintf( outfile, "   }\n" );

		(void) fprintf(outfile, "// Cross ends of cone\n" );
		fprintf( outfile, "#declare %s_cone_cross_rightend%d = union {\n", 
			ProgName, FrameId );
		link1 = linkhead2;
		for (i=1; i<FramePoints2+1, link1!=NULL; i++ )
		{
			/* Compute the offset joint point */
			if ( (j = i + FrameCrossOffset) > FramePoints2 )
			{
				j = j - FramePoints2;
			}
			k = 1;
			link2 = linkhead2;
			while ( k < j )
			{
				link2 = link2->next;
				k++;
			}
			
			fprintf ( outfile, "     cylinder { <%f,%f,%f>, <%f,%f,%f>, %f } \n",
					link1->x, link1->y, link1->z, link2->x, link2->y, link2->z,
					FrameRodRadius);
			link1 = link1->next;
		}
		fprintf( outfile, "   }\n" );
	}


	/*
	 * Join the ends across the length, if requested.
	 * Remember, we forced the number of points on the ends to be equal
	 * if joining the length was requested.
	 */
	if ( FrameJoinLength )
	{
		(void) fprintf(outfile, "// Cross ends of cone\n" );
		fprintf( outfile, "#declare %s_cone_join_length%d = union {\n", 
			ProgName, FrameId );
		link1 = linkhead1;
		link2 = linkhead2;
		for (i=0; i<FramePoints1, link1!=NULL; i++ )
		{
			fprintf ( outfile, "     cylinder { <%f,%f,%f>, <%f,%f,%f>, %f } \n",
					link1->x, link1->y, link1->z, link2->x, link2->y, link2->z,
					FrameRodRadius);
			link1 = link1->next;
			link2 = link2->next;
		}
		fprintf( outfile, "   }\n" );
	}


	/* Cross the length, if requested */
	if ( FrameCrossLength )
	{
		(void) fprintf(outfile, "// Cross length of cone\n" );
		fprintf( outfile, "#declare %s_cone_cross_length%d = union {\n", 
			ProgName, FrameId );
		link1 = linkhead1;
		link2 = linkhead2;
		for (i=1; i<FramePoints1+1, link1!=NULL; i++ )
		{
			/* Compute the offset for the right end of the cone */
			if ( (j = i + FrameCrossOffset) > FramePoints2 )
			{
				j = j - FramePoints2;
			}
			k = 1;
			link2 = linkhead2;
			while ( k < j )
			{
				link2 = link2->next;
				k++;
			}
			
			fprintf ( outfile, "     cylinder { <%f,%f,%f>, <%f,%f,%f>, %f } \n",
					link1->x, link1->y, link1->z, link2->x, link2->y, link2->z,
					FrameRodRadius);
			link1 = link1->next;
		}
		fprintf( outfile, "   }\n" );
	}

	/*
	 * Combine all pieces into a single union
	 */
	if ( FrameElements == 1 )
	{
		(void) fprintf(outfile, "// Combined cone\n" );
		fprintf( outfile, "#declare %s_cone%d = union {\n", ProgName, FrameId );
	}
	else
	{
		(void) fprintf(outfile, "// single cone element \n" );
		fprintf( outfile, "#declare %s_cone_element%d = union {\n", ProgName, FrameId );
	}
	fprintf( outfile, "   object { %s_cone_rims%d }\n", ProgName, FrameId );
	if ( FrameCrossEnds )
	{
		fprintf( outfile, "   object { %s_cone_cross_leftend%d }\n", ProgName, FrameId );
		fprintf( outfile, "   object { %s_cone_cross_rightend%d }\n", ProgName, FrameId );
	}
	if ( FrameJoinLength )
		fprintf( outfile, "   object { %s_cone_join_length%d }\n", ProgName, FrameId );
	if ( FrameCrossLength )
		fprintf( outfile, "   object { %s_cone_cross_length%d }\n", ProgName, FrameId );


	/* 
	 * Add the Joint Objects to the vertices, if requested.
	 */
	if ( strcmp ( FrameJointObject, GENF_DEFAULT_JOINT_OBJECT ) != 0 )
	{
		(void) fprintf(outfile, "// Add Joint Objects to the vertices\n" );

		if ( FrameCrossLength )
		{
			link1 = linkhead1;
			link2 = linkhead2;
			for ( i=0; i<FramePoints1; i++)
			{
				fprintf( outfile, "   object { %s translate <%f, %f, %f> }\n", 
							FrameJointObject, link1->x, link1->y, link1->z );
				fprintf( outfile, "   object { %s translate <%f, %f, %f> }\n", 
							FrameJointObject, link2->x, link2->y, link2->z );
				link1 = link1->next;
				link2 = link2->next;
			}
		}
		else
		{
			link1 = linkhead1;
			for ( i=0; i<FramePoints1; i++)
			{
				fprintf( outfile, "   object { %s translate <%f, %f, %f> }\n", 
							FrameJointObject, link1->x, link1->y, link1->z );
				link1 = link1->next;
			}
			link1 = linkhead2;
			for ( i=0; i<FramePoints2; i++)
			{
				fprintf( outfile, "   object { %s translate <%f, %f, %f> }\n", 
							FrameJointObject, link1->x, link1->y, link1->z );
				link1 = link1->next;
			}
		}
	}


	/* Texture it, if its the only element */
	if ( FrameElements == 1 )
		fprintf( outfile, "   texture { %s }\n", FrameTexture );
	
	fprintf( outfile, "   }\n" );

	/*
	 * if more than one element has been requested, make a union
	 * of multiple elements
	 */
	if ( FrameElements > 1 )
	{
		(void) fprintf(outfile, "// Combined cone\n" );
		fprintf( outfile, "#declare %s_cone%d = union {\n", ProgName, FrameId );

		/* compute the translation offsets */
		for (i=FrameElements; i>0; i-=2)
		{
			/* put an element in the union at the computed offset */
			xoffset = (double)(i-1.0)/2.0;
			fprintf( outfile, 
				"   object { %s_cone_element%d translate <%f, 0, 0>}\n", 
						ProgName, FrameId, (double)(-1.0*xoffset*FrameLength) );
			fprintf( outfile, 
				"   object { %s_cone_element%d translate <%f, 0, 0>}\n", 
						ProgName, FrameId, (double)(xoffset*FrameLength) );
		}

		/* texture the complete object */
		fprintf( outfile, "   texture { %s }\n", FrameTexture );
		fprintf( outfile, "   }\n" );
	}

	(void) fclose(outfile);
}

