/* $Id$ */

/*========================================================================
 *  Copyright (c) 1996 Michael J. Hammel <mjhammel@csn.net>
 *========================================================================
 *              FILE NAME: genf.c
 *            DESCRIPTION: generates frame objects for POVRay scenes
 *      DEFINED CONSTANTS: 
 *       TYPE DEFINITIONS: 
 *      MACRO DEFINITIONS: 
 *       GLOBAL VARIABLES: 
 *       PUBLIC FUNCTIONS: 
 *      PRIVATE FUNCTIONS: 
 *  SOFTWARE DEPENDENCIES: 
 *  HARDWARE DEPENDENCIES: 
 *                  NOTES: 
 *
 * SPECIAL CONSIDERATIONS:
 * Set tabstops to 3 to make this file more readable.
 *
 *========================================================================
 *
 * MODIFICATION HISTORY:
 * $Log$
 *
 *========================================================================
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 *========================================================================*/

/* === System Headers === */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>


/* === Project Headers === */
#include "genf.h"

/* === external routines === */
extern char *utilGetWordA(char *line, char stop);
extern char *utilGetValue(char *name, struct entry *head);
extern int  utilIsNumber(char *string);
extern int  utilIsFloat(char *string);
extern char *utilStrupr(char *string);
extern int  utilReadConfigFile( char *config_file, struct entry **head );
extern void BuildBox();

/* === Public routine prototypes === */

/* === Private routine prototypes === */
void BuildCylinder();
void BuildCone();

/* === Global Variables === */
extern char *utilError;
char ErrorBuf[255];

char	ProgName[255];
char	OutFileName[128];
FILE	*outfile;

/* character strings */
char	FrameShape[128], FrameJointObject[128], FrameTexture[128];
char	FrameConeEndShape[128];

/* numeric values */
int		FrameId, FrameElements, FramePoints1, FramePoints2,
			FrameCrossOffset;
double	FrameRadius1, FrameRadius2, FrameLength, 
			FrameBoxWidth, FrameBoxHeight, FrameRodRadius;
double	FrameConeEndThickness;

/* booleans */
int	FrameCrossEnds, FrameCrossLength, FrameJoinLength;

/* configuration file info */
struct entry *ConfigHead;
char ConfigFile[128];
char *GENF_FRAME_ID;
char *GENF_FRAME_ROD_RADIUS;
char *GENF_FRAME_SHAPE;
char *GENF_FRAME_JOINT_OBJECT;
char *GENF_FRAME_TEXTURE;
char *GENF_FRAME_ELEMENTS;
char *GENF_FRAME_RADIUS1;
char *GENF_FRAME_RADIUS2;
char *GENF_FRAME_POINTS1;
char *GENF_FRAME_POINTS2;
char *GENF_FRAME_LENGTH;
char *GENF_FRAME_CROSS_OFFSET;
char *GENF_FRAME_BOX_WIDTH;
char *GENF_FRAME_BOX_HEIGHT;
char *GENF_FRAME_CROSS_ENDS;
char *GENF_FRAME_CROSS_LENGTH;
char *GENF_FRAME_JOIN_LENGTH;
char *GENF_FRAME_OUTFILENAME;
char *GENF_CONE_END_SHAPE;
char *GENF_CONE_END_THICKNESS;



/*========================================================================
 *	Name:			ParseCmdLine
 *	Prototype:	int ParseCmdLine( int argc, char **argv )
 *					
 *
 *	Description:
 *		parses command line options into global variables using getopt()
 *
 *	Input Arguments:
 *		int argc			number of command line arguments
 *		char **argv		array of command line argument strings
 *		
 *	Return Values:
 *		int optind		index of next command line argument to process
 *
 *	Global Variables:
 *
 *	Notes:
 *		
 *========================================================================*/

int 
ParseCmdLine(
	int argc,
	char **argv
)
{
	int c;

	opterr=0;
	while ( ( c = getopt( argc, argv, "?c:" ) ) != -1 )
   {
   switch (c)
      {
      case 'c':						/* Output file name */
				(void) strcpy ( ConfigFile, optarg );
				break;

      case '?': 						/* usage statement */
				printf("%s: usage:\n", argv[0]);
				printf( "%s\n\n", USAGE );
				exit (0);
				break;
      default:	
				printf("%s: usage:\n", argv[0]);
				printf( "%s\n\n", USAGE );
				exit (0);
				break;
      }
   }
	return(optind);
}


/*========================================================================
 *	Name:			Validate
 *	Prototype:	int Validate()
 *					
 *
 *	Description:
 *		validates the tags used in the configuration file
 *
 *	Input Arguments:
 *		struct engtry *head		points to the head of the entry link list
 *		
 *	Return Values:
 *		int 		0: all values are ok
 *		         1: an invalid configuration item has been encountered
 *
 *	Global Variables:
 *		ErrorBuf	error message describing invalid configuation item.
 *
 *	Notes:
 *		Ugly, but since when is text processing pretty?  Can't we all 
 *		just read 1's and 0's!
 *
 *========================================================================*/

int 
Validate()
{

	if ( GENF_FRAME_ID != NULL )
		if ( utilIsNumber(GENF_FRAME_ID) == 0 )
			FrameId = atoi(GENF_FRAME_ID);
		else
		{
			sprintf(ErrorBuf,
				"Bad value %s for %s\n", GENF_FRAME_ID, GENF_TAG_ID );
			return (1);
		}

	if ( GENF_FRAME_ROD_RADIUS != NULL )
		if ( utilIsFloat(GENF_FRAME_ROD_RADIUS) == 0 )
			FrameRodRadius = atof(GENF_FRAME_ROD_RADIUS);
		else
		{
			sprintf(ErrorBuf,
				"Bad value %s for %s\n", GENF_FRAME_ROD_RADIUS, GENF_TAG_ROD_RADIUS );
			return (1);
		}

	if ( GENF_FRAME_ELEMENTS != NULL )
		if ( utilIsNumber(GENF_FRAME_ELEMENTS) == 0 )
			FrameElements = atoi(GENF_FRAME_ELEMENTS);
		else
		{
			sprintf(ErrorBuf,
				"Bad value %s for %s\n", GENF_FRAME_ELEMENTS, GENF_TAG_ELEMENTS );
			return (1);
		}

	if ( GENF_FRAME_LENGTH != NULL )
		if ( utilIsFloat(GENF_FRAME_LENGTH) == 0 )
			FrameLength = atof(GENF_FRAME_LENGTH);
		else
		{
			sprintf(ErrorBuf,
				"Bad value %s for %s\n", GENF_FRAME_LENGTH, GENF_TAG_LENGTH );
			return (1);
		}

	if ( GENF_FRAME_POINTS1 != NULL )
		if ( utilIsNumber(GENF_FRAME_POINTS1) == 0 )
			FramePoints1 = atoi(GENF_FRAME_POINTS1);
		else
		{
			sprintf(ErrorBuf,
				"Bad value %s for %s\n", GENF_FRAME_POINTS1, GENF_TAG_POINTS1 );
			return (1);
		}


	if ( GENF_FRAME_POINTS2 != NULL )
		if ( utilIsNumber(GENF_FRAME_POINTS2) == 0 )
			FramePoints2 = atoi(GENF_FRAME_POINTS2);
		else
		{
			sprintf(ErrorBuf,
				"Bad value %s for %s\n", GENF_FRAME_POINTS2, GENF_TAG_POINTS1 );
			return (1);
		}

	if ( GENF_FRAME_CROSS_OFFSET != NULL )
		if ( utilIsNumber(GENF_FRAME_CROSS_OFFSET) == 0 )
			FrameCrossOffset = atoi(GENF_FRAME_CROSS_OFFSET);
		else
		{
			sprintf(ErrorBuf,
				"Bad value %s for %s\n", GENF_FRAME_CROSS_OFFSET, GENF_TAG_CROSS_OFFSET );
			return (1);
		}

	if ( GENF_FRAME_RADIUS1 != NULL )
		if ( utilIsFloat(GENF_FRAME_RADIUS1) == 0 )
			FrameRadius1 = atof(GENF_FRAME_RADIUS1);
		else
		{
			sprintf(ErrorBuf,
				"Bad value %s for %s\n", GENF_FRAME_RADIUS1, GENF_TAG_RADIUS1 );
			return (1);
		}

	if ( GENF_FRAME_RADIUS2 != NULL )
		if ( utilIsFloat(GENF_FRAME_RADIUS2) == 0 )
			FrameRadius2 = atof(GENF_FRAME_RADIUS2);
		else
		{
			sprintf(ErrorBuf,
				"Bad value %s for %s\n", GENF_FRAME_RADIUS2, GENF_TAG_RADIUS2 );
			return (1);
		}

	if ( GENF_FRAME_BOX_WIDTH != NULL )
		if ( utilIsFloat(GENF_FRAME_BOX_WIDTH) == 0 )
			FrameBoxWidth = atof(GENF_FRAME_BOX_WIDTH);
		else
		{
			sprintf(ErrorBuf,
				"Bad value %s for %s\n", GENF_FRAME_BOX_WIDTH, GENF_TAG_BOX_WIDTH );
			return (1);
		}

	if ( GENF_FRAME_BOX_HEIGHT != NULL )
		if ( utilIsFloat(GENF_FRAME_BOX_HEIGHT) == 0 )
			FrameBoxHeight = atof(GENF_FRAME_BOX_HEIGHT);
		else
		{
			sprintf(ErrorBuf,
				"Bad value %s for %s\n", GENF_FRAME_BOX_HEIGHT, GENF_TAG_BOX_HEIGHT );
			return (1);
		}
	if ( GENF_CONE_END_THICKNESS != NULL )
		if ( utilIsFloat(GENF_CONE_END_THICKNESS) == 0 )
			FrameConeEndThickness = atof(GENF_CONE_END_THICKNESS);
		else
		{
			sprintf(ErrorBuf,
				"Bad value %s for %s\n", 
					GENF_CONE_END_THICKNESS, GENF_TAG_CONE_END_THICKNESS );
			return (1);
		}




	if ( GENF_FRAME_CROSS_ENDS != NULL )
	{
		utilStrupr(GENF_FRAME_CROSS_ENDS);
		if ( strcmp ( "FALSE", GENF_FRAME_CROSS_ENDS ) == 0 )
			FrameCrossEnds = False;	/* don't put frame member on ends */
		else
			FrameCrossEnds = True;
	}

	if ( GENF_FRAME_CROSS_LENGTH != NULL )
	{
		utilStrupr(GENF_FRAME_CROSS_LENGTH);
		if ( strcmp ( "TRUE", GENF_FRAME_CROSS_LENGTH ) == 0 )
			FrameCrossLength = True;	/* do put frame members along body */
		else
			FrameCrossLength = False;
	}

	if ( GENF_FRAME_JOIN_LENGTH != NULL )
	{
		utilStrupr(GENF_FRAME_JOIN_LENGTH);
		if ( strcmp ( "TRUE", GENF_FRAME_JOIN_LENGTH ) == 0 )
			FrameJoinLength = True;		/* do put frame members on box edges */
		else
			FrameJoinLength = False;
	}



	if ( GENF_FRAME_JOINT_OBJECT != NULL )
		(void) strcpy( FrameJointObject, GENF_FRAME_JOINT_OBJECT);

	if ( GENF_FRAME_SHAPE != NULL )
	{
		utilStrupr ( GENF_FRAME_SHAPE );
		(void) strcpy( FrameShape, GENF_FRAME_SHAPE);
	}

	if ( GENF_FRAME_TEXTURE != NULL )
		(void) strcpy( FrameTexture, GENF_FRAME_TEXTURE);

	if ( GENF_FRAME_OUTFILENAME != NULL )
		(void) strcpy( OutFileName, GENF_FRAME_OUTFILENAME );

	if ( GENF_CONE_END_SHAPE != NULL )
	{
		utilStrupr ( GENF_CONE_END_SHAPE );
		(void) strcpy( FrameConeEndShape, GENF_CONE_END_SHAPE);
	}


	/*
	 * Check that shape request is valid
	 */
	if (	( strcmp (FrameShape, "BOX") != 0 ) &&
			( strcmp (FrameShape, "CONE") != 0 ) &&
			( strcmp (FrameShape, "CYLINDER") != 0 ) )
	{
			sprintf(ErrorBuf,
				"%s is not a valid frame shape", FrameShape );
			return (1);
	}
	if (	( strcmp (FrameConeEndShape, "TORUS") != 0 ) &&
			( strcmp (FrameConeEndShape, "CYLINDER") != 0 ) )
	{
			sprintf(ErrorBuf,
				"%s is not a valid cone end shape", FrameConeEndShape );
			return (1);
	}
	

	/*
	 * Check that various values are non-negative
	 */
	if ( FrameRodRadius < 0.0 )
	{
			sprintf(ErrorBuf,
				"%f is not a valid value for the rod radius", FrameRodRadius );
			return (1);
	}
	if ( FrameLength < 0.0 )
	{
			sprintf(ErrorBuf,
				"%f is not a valid value for the frame length", FrameLength );
			return (1);
	}
	if ( FrameRadius1 < 0.0 )
	{
			sprintf(ErrorBuf,
				"%f is not a valid value for radius1", FrameRadius1 );
			return (1);
	}
	if ( FrameRadius2 < 0.0 )
	{
			sprintf(ErrorBuf,
				"%f is not a valid value for radius2", FrameRadius2 );
			return (1);
	}
	if ( FramePoints1 < 0 )
	{
			sprintf(ErrorBuf,
				"%d is not a valid value for points1", FramePoints1 );
			return (1);
	}
	if ( FramePoints2 < 0 )
	{
			sprintf(ErrorBuf,
				"%d is not a valid value for points2", FramePoints2 );
			return (1);
	}
	if ( FrameElements < 0 )
	{
			sprintf(ErrorBuf,
				"%d is not a valid value for the number of elements", FrameElements );
			return (1);
	}
	if ( FrameCrossOffset < 0 )
	{
			sprintf(ErrorBuf,
				"%d is not a valid value for the cross offset", FrameCrossOffset );
			return (1);
	}
	if ( FrameConeEndThickness < 0.0 )
	{
			sprintf(ErrorBuf,
				"%f is not a valid value for coneendthickness", FrameConeEndThickness );
			return (1);
	}
	if ( FrameBoxHeight < 0.0 )
	{
			sprintf(ErrorBuf,
				"%f is not a valid value for the box height", FrameBoxHeight );
			return (1);
	}
	if ( FrameBoxWidth < 0.0 )
	{
			sprintf(ErrorBuf,
				"%f is not a valid value for the box width", FrameBoxWidth );
			return (1);
	}
	

	return (0);
}


/*========================================================================
 *	Name:			Main
 *========================================================================*/

void
main(
	int	argc,
	char	**argv
)
{

	/* 
	 * Initialization
	 */

	strcpy( ProgName, argv[0] );
	strcpy( ConfigFile, GENF_DEFAULT_CONFIG_FILE );

	FrameId          = GENF_DEFAULT_ID;				/* default id */
	FrameRodRadius   = GENF_DEFAULT_ROD_RADIUS;	/* default rod radius */
	FrameElements    = GENF_DEFAULT_ELEMENTS;		/* default # of elements */
	FrameLength      = GENF_DEFAULT_LENGTH;		/* default length of 1 element */
	FramePoints1     = GENF_DEFAULT_POINTS;		/* default # of points on end */
	FramePoints2     = GENF_DEFAULT_POINTS;		/* default # of points on end */
	FrameCrossOffset = GENF_DEFAULT_CROSS_OFFSET;/* default end points offset */
	FrameRadius1     = GENF_DEFAULT_RADIUS;		/* default radius of end */
	FrameRadius2     = GENF_DEFAULT_RADIUS;		/* default radius of end */
	FrameBoxWidth    = GENF_DEFAULT_BOX_WIDTH;	/* default box width */
	FrameBoxHeight   = GENF_DEFAULT_BOX_HEIGHT;	/* default box height */

	/* default thickness for cone end shapes */
	FrameConeEndThickness = GENF_DEFAULT_CONE_END_THICKNESS;	

	FrameCrossEnds   = False;			/* don't put frame member on ends */
	FrameCrossLength = True;			/* do put frame members along body */
	FrameJoinLength  = True;			/* do put frame members on box edges */

	(void) strcpy( FrameJointObject, GENF_DEFAULT_JOINT_OBJECT);
	(void) strcpy( FrameShape, GENF_DEFAULT_SHAPE);
	(void) strcpy( FrameTexture, GENF_DEFAULT_TEXTURE);
	(void) strcpy( OutFileName, GENF_DEFAULT_OUTFILENAME );
	(void) strcpy( FrameConeEndShape, GENF_DEFAULT_CONE_END_SHAPE );


	/* 
	 * Parse command line arguments
	 */
	ParseCmdLine(argc, argv);


	/*
	 * Read the configuration file, then validate the entries
	 */
	if ( utilReadConfigFile(ConfigFile, &ConfigHead) != 0 )
	{
		printf("%s: %s (%s:%d)\n", ProgName, utilError, __FILE__, __LINE__);
		exit(1);
	}

	GENF_FRAME_ID = utilGetValue ( GENF_TAG_ID , ConfigHead );
	GENF_FRAME_ROD_RADIUS = utilGetValue ( GENF_TAG_ROD_RADIUS , ConfigHead );
	GENF_FRAME_SHAPE = utilGetValue ( GENF_TAG_SHAPE , ConfigHead );
	GENF_FRAME_JOINT_OBJECT = utilGetValue ( GENF_TAG_JOINT_OBJECT , ConfigHead );
	GENF_FRAME_TEXTURE = utilGetValue ( GENF_TAG_TEXTURE , ConfigHead );
	GENF_FRAME_ELEMENTS = utilGetValue ( GENF_TAG_ELEMENTS , ConfigHead );
	GENF_FRAME_RADIUS1 = utilGetValue ( GENF_TAG_RADIUS1 , ConfigHead );
	GENF_FRAME_RADIUS2 = utilGetValue ( GENF_TAG_RADIUS2 , ConfigHead );
	GENF_FRAME_POINTS1 = utilGetValue ( GENF_TAG_POINTS1 , ConfigHead );
	GENF_FRAME_POINTS2 = utilGetValue ( GENF_TAG_POINTS2 , ConfigHead );
	GENF_FRAME_LENGTH = utilGetValue ( GENF_TAG_LENGTH , ConfigHead );
	GENF_FRAME_CROSS_OFFSET = utilGetValue ( GENF_TAG_CROSS_OFFSET , ConfigHead );
	GENF_FRAME_BOX_WIDTH = utilGetValue ( GENF_TAG_BOX_WIDTH , ConfigHead );
	GENF_FRAME_BOX_HEIGHT = utilGetValue ( GENF_TAG_BOX_HEIGHT , ConfigHead );
	GENF_FRAME_CROSS_ENDS = utilGetValue ( GENF_TAG_CROSS_ENDS , ConfigHead );
	GENF_FRAME_CROSS_LENGTH = utilGetValue ( GENF_TAG_CROSS_LENGTH , ConfigHead );
	GENF_FRAME_JOIN_LENGTH = utilGetValue ( GENF_TAG_JOIN_LENGTH , ConfigHead );
	GENF_FRAME_OUTFILENAME = utilGetValue ( GENF_TAG_OUTFILENAME , ConfigHead );
	GENF_CONE_END_SHAPE = utilGetValue ( GENF_TAG_CONE_END_SHAPE , ConfigHead );
	GENF_CONE_END_THICKNESS = utilGetValue ( GENF_TAG_CONE_END_THICKNESS , ConfigHead );

	if ( Validate() != 0 )
	{
		printf("%s: %s (%s:%d)\n", ProgName, ErrorBuf, __FILE__, __LINE__);
		exit(1);
	}


	/* 
	 * print parameters being used for this run
	 */
	printf("\nGenerating Frame:\n");
	printf("    Output File       : %s \n", OutFileName);
	printf("    ID                : %d \n", FrameId);
	printf("    Frame Shape       : %s \n", FrameShape);
	printf("    Cone End Shape    : %s \n", FrameConeEndShape);
	printf("    Cone End Thickness: %f \n", FrameConeEndThickness);
	printf("    Number of Elements: %d \n", FrameElements);
	printf("    Rod Radius        : %f \n", FrameRodRadius);
	printf("    Joint Object      : %s \n", FrameJointObject);
	printf("    Texture           : %s \n", FrameTexture);
	printf("    Cross Ends        : %d (0:no 1:yes)\n", FrameCrossEnds);
	printf("    Cross Length      : %d (0:no 1:yes)\n", FrameCrossLength);
	printf("    Join Length       : %d (0:no 1:yes)\n", FrameJoinLength);
	printf("    Points 1          : %d \n", FramePoints1);
	printf("    Points 2          : %d \n", FramePoints2);
	printf("\n\n");
	printf(
		"    Frame length  : %.4f    Cross Points Offset: %4d\n" 
		"    Radius 1      : %.4f    Radius 2           : %.4f\n"
		"    Box Width     : %.4f    Box Height         : %.4f\n",
		FrameLength, FrameCrossOffset,
		FrameRadius1, FrameRadius2,
		FrameBoxWidth, FrameBoxHeight);

	printf("______________________________________________________________\n\n");


	/* ------------------------------------------------*/
	/* ---- Check if the file can be opened    --------*/
	/* ------------------------------------------------*/

	if ((outfile = fopen(OutFileName,"w")) == NULL)
	{
		(void) printf("%s:  Error: Unable to open %s for writing \n",
					argv[0], OutFileName) ;
		exit ( 1 );
	}


	/* ------------------------------------------------*/
	/* ---- Generate elements                    ------*/
	/* ------------------------------------------------*/

	utilStrupr(FrameShape);
	if ( strcmp ( FrameShape, "BOX" ) == 0 )
		BuildBox();
	if ( strcmp ( FrameShape, "CYLINDER" ) == 0 )
		BuildCone();
	if ( strcmp ( FrameShape, "CONE" ) == 0 )
		BuildCone();

	/*
	 * all done
	 */
	exit(0);
}


/*========================================================================
 *	Name:			PrintHeader
 *	Prototype:	void PrintHeader( char *string )
 *
 *	Description:
 *		prints the header information in the output file
 *
 *	Input Arguments:
 *		char *string		string based on which type of frame is being
 *								generated
 *		
 *	Return Values:
 *
 *	Notes:
 *
 *========================================================================*/

void
PrintHeader( char *string )
{

	(void) fprintf(outfile, "// This file generated with the genf utility.\n");
	(void) fprintf(outfile, "// %s\n//\n", COPYRIGHT);
	(void) fprintf(outfile, "// %s\n", string);
	(void) fprintf(outfile, "//\n" );
}

