/* $Id$ */

/*========================================================================
 * Copyright (c) 1996 Michael J. Hammel <mjhammel@csn.net>
 *========================================================================
 *              FILE NAME: util.c
 *            DESCRIPTION: various utilities used by genf
 *
 * NOTES: 
 *
 * SPECIAL CONSIDERATIONS:
 *	Set tabstops to 3 for readability
 *========================================================================
 *
 * MODIFICATION HISTORY:
 * $Log$
 *
 *========================================================================*/

/* === System Headers === */
#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>
#include <ctype.h>
#include <string.h>
#include <time.h>

/*
#include <sys/param.h>
#include <pwd.h>
*/

/* === Project Headers === */
#include "genf.h"

/* === external routines === */

/* === Public routine prototypes === */

/* === Private routine prototypes === */

/* === Global Variables === */
char	utilError[255];		/* error messages from util routines */


/*========================================================================
 *	Name     : utilStrupr
 *	Prototype: void utilStrupr (char *line)
 *
 *	Description:
 *		Takes a character string and converts it to uppercase.
 *
 *	Input Arguments:
 *		char	*line			character string to convert
 *
 *	Return Value:
 *
 * Notes:
 *		converts the string in place
 *		
 *========================================================================*/
void
utilStrupr(
	char *line 
)
{
	int	i;

	for (i=0;i<strlen(line);i++)
		*(line+i) = toupper(line[i]);    
}



/*========================================================================
 *	Name     : utilIsNumber
 *	Prototype: int utilIsNumber (char *line)
 *
 *	Description:
 *		Checks that a character string contains nothing but digits
 *
 *	Input Arguments:
 *		char	*line			character string to check
 *
 *	Return Value:
 *		int	0 if the string is numeric, 1 if its not
 *
 * Notes:
 *		
 *========================================================================*/
int
utilIsNumber(
	char *line 
)
{
	int	i;

	for (i=0;i<strlen(line);i++)
		if ( isdigit((int)*(line+i)) == 0 )
			return (1);
	return(0);
}



/*========================================================================
 *	Name     : utilIsFloat
 *	Prototype: int utilIsFloat (char *line)
 *
 *	Description:
 *		Checks that a character string is a valid float value
 *
 *	Input Arguments:
 *		char	*line			character string to check
 *
 *	Return Value:
 *		int	0 if the string is a float value, 1 if its not
 *
 * Notes:
 *		
 *========================================================================*/
int
utilIsFloat(
	char *line 
)
{
	int	i;

	for (i=0;i<strlen(line);i++)
		if ( isdigit((int)*(line+i)) == 0 )
			if ( strncmp((char *)(line+i), ".", 1) != 0 )
				return (1);
	return(0);
}



/*========================================================================
 *	Name     : utilGetWordA			 
 *	Prototype: char *utilGetWordA(char *line, char stop)
 *
 *	Description:
 *		Parse a word, delimited by the stop character, from line.  Update
 *		line to point to character immediately after stop character.
 *		Allocates the storage for the parsed word.
 *
 *	Input Arguments:
 *		char *line				line to parse word from
 *		char stop				character to use as the delimiter between words
 *
 *	Return Values:
 *		char *word				points to word parsed by utilGetWordA()
 *
 *	Restrictions:
 *		caller is responsible for cleaning up storage allocated by this
 *		routine.
 *		
 *========================================================================*/
char *
utilGetWordA(
	char *line, 
	char stop
)
{
    int x = 0,y;
    char *word = (char *) malloc(sizeof(char) * (strlen(line) + 1));

    for(x=0;((line[x]) && (line[x] != stop));x++)
        word[x] = line[x];

    word[x] = '\0';
    if(line[x]) ++x;
    y=0;

    while( (line[y++] = line[x++]) );
    return word;
}


/*========================================================================
 *	Name     : utilReadConfigFile
 *	Prototype: int utilReadConfigFile( char *config_file, struct entry **head )
 *					
 *	Description:
 *		read in the config file into an entry link list
 *		Config file lines have the format
 *			keyword:value
 *		Comments are lines which begin with ##
 *		Blank lines and leading white space is ignored.
 *
 *	Input Arguments:
 *		config_file			name of configuration file to read
 *
 *	Output Arguments:
 *		entry	**head		points to head of link list when complete
 *		
 *========================================================================*/

int
utilReadConfigFile(
	char				*config_file,
	struct entry	**head
)
{
	struct stat		stat_buf;
	struct entry	*entries = NULL;

	char	line[255], *chr;
	FILE	*fd;

	/*
	 * Make sure config file exists
	 */
	if ( stat( config_file, &stat_buf ) == -1 )
	{
		sprintf ( utilError, "Can't find config file %s. Aborting.\n", config_file );
		return ( -1 );
	}

	/* 
	 * read in the config information
	 */
	fd = fopen ( config_file, "r" );
		
	while ( fgets( line, 255, fd ) != NULL )
	{
		/*
		 * If the line is not a comment...
		 */
		if ( strncmp( line, "##", 2 ) != 0 )
		{
			/*
			 * Move past white space
			 */
			chr=line;
			while (	(isspace ((int)*chr) != 0 ) && 
						(chr != (line+strlen(line)) ) )
				chr++;
			if (chr != (line+strlen(line)) )
			{

				/*
				 * remove the newline
				 */
				strncpy ( (char *)(line+strlen(line)-1), "\0", 1 );
	
				/*
				 * Get an entry struct
				 */
				if (entries)
				{ 
					entries->next = (struct entry *) malloc(sizeof(struct entry));
					entries=entries->next;
				} 
				else
				{ 
					entries = (struct entry *) malloc(sizeof(struct entry));
					*head = entries;
				}
		
				/*
				 * Pull the keyword and value from the line
				 */
				entries->name = utilGetWordA(chr, DELIMITER );
				/* entries->val = utilGetWordA( chr, DELIMITER );
				 */
				if ( strlen ( chr ) > 0 )
				{
					entries->val= (char *)malloc(strlen(chr)+1);
					strcpy(entries->val, chr);
				}
				else
					entries->val=NULL;
		
				entries->next=NULL;
		
			}
		}
	}

	return(0);
}



/*========================================================================
 *	Name     : utilGetValue
 *	Prototype: char *utilGetValue(char *name, struct entry *head )
 *					
 *	Description:
 *		retrieve the value for the given field from the link list of
 *		name/value pairs
 *
 *	Input Arguments:
 *		char				*name		name of name/value pair to look up
 *		struct entry	*head		head of link list
 *
 *	Return Values:
 *		char		*					value string for that name
 *		
 *========================================================================*/

/*
 * Get the value associated with the name thats been requested
 */
char *
utilGetValue(
	char *name,
	struct entry *head
)
{
	struct entry *entries;

	entries = head;
	while ( entries )
	{
		if ( strcmp ( entries->name, name ) == 0 )
		{
			return ( entries->val );
		}
		entries = entries->next;
	}
	return ( (char *)NULL );

}


/*========================================================================
 *	Name     : utilGetDate
 *	Prototype: int utilGetDate()
 *					
 *	Description:
 *		compute todays date and convert it to a date of the format
 *		"yyyymmdd".  In this way the integer values can be sorted correctly.
 *
 *	Return Values:
 *		int	date in integer format
 *
 *	Notes:
 *========================================================================*/
int
utilGetDate()
{
	time_t		timep;
	struct tm	*timeptr;
	char			timestr[10];

	timep = time(NULL);
	timeptr = localtime(&timep);
	sprintf ( timestr, "%d%02d%02d", 
			timeptr->tm_year+1900, timeptr->tm_mon+1, timeptr->tm_mday );
	return(atoi(timestr));
}

